@@ -40,7 +40,7 @@
 #include "gconf-property-editor.h"
 
 
-#define DEFAULT_APPS_KEY_BROWSER_PATH "/desktop/gnome/url-handlers/unknown"
+#define DEFAULT_APPS_KEY_BROWSER_PATH "/desktop/gnome/url-handlers/http"
 #define DEFAULT_APPS_KEY_BROWSER_NEEDS_TERM DEFAULT_APPS_KEY_BROWSER_PATH"/needs_terminal"
 #define DEFAULT_APPS_KEY_BROWSER_EXEC       DEFAULT_APPS_KEY_BROWSER_PATH"/command"
 
@@ -48,11 +48,6 @@
 #define DEFAULT_APPS_KEY_MAILER_NEEDS_TERM DEFAULT_APPS_KEY_MAILER_PATH"/needs_terminal"
 #define DEFAULT_APPS_KEY_MAILER_EXEC       DEFAULT_APPS_KEY_MAILER_PATH"/command"
 
-#define DEFAULT_APPS_KEY_HELP_VIEWER_PATH "/desktop/gnome/applications/help_viewer"
-#define DEFAULT_APPS_KEY_HELP_VIEWER_NEEDS_TERM DEFAULT_APPS_KEY_HELP_VIEWER_PATH"/needs_term"
-#define DEFAULT_APPS_KEY_HELP_VIEWER_ACCEPTS_URLS DEFAULT_APPS_KEY_HELP_VIEWER_PATH"/accepts_urls"
-#define DEFAULT_APPS_KEY_HELP_VIEWER_EXEC DEFAULT_APPS_KEY_HELP_VIEWER_PATH"/exec"
-
 #define DEFAULT_APPS_KEY_TERMINAL_PATH "/desktop/gnome/applications/terminal"
 #define DEFAULT_APPS_KEY_TERMINAL_EXEC_ARG DEFAULT_APPS_KEY_TERMINAL_PATH"/exec_arg"
 #define DEFAULT_APPS_KEY_TERMINAL_EXEC DEFAULT_APPS_KEY_TERMINAL_PATH"/exec"
@@ -61,7 +56,6 @@
 
 typedef struct _BrowserDescription BrowserDescription;
 typedef struct _MailerDescription MailerDescription;
-typedef struct _HelpViewDescription HelpViewDescription;
 typedef struct _TerminalDesciption TerminalDescription;
 
 /* All defined below */
@@ -85,7 +79,7 @@ on_text_custom_properties_clicked (GtkWidget *w, GladeXML *dialog)
 	mime_app = gnome_vfs_application_registry_get_mime_application (MIME_APPLICATION_ID);
 
 	gtk_entry_set_text (GTK_ENTRY (WID ("text_custom_name_entry")),
-			    mime_app ? mime_app->name : "");
+			    mime_app ? _(mime_app->name) : "");
 	gtk_entry_set_text (GTK_ENTRY (WID ("text_custom_command_entry")),
 			    mime_app ? mime_app->command : "");
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("text_custom_multi_toggle")),
@@ -123,7 +117,7 @@ on_text_custom_properties_clicked (GtkWidget *w, GladeXML *dialog)
 		g_free (mime_app->name);
 		g_free (mime_app->command);
 	} else {
-		mime_app = g_new0 (GnomeVFSMimeApplication, 1);	
+		mime_app = g_new0 (GnomeVFSMimeApplication, 1);
 		mime_app->id = g_strdup (MIME_APPLICATION_ID);
 	}
 
@@ -145,7 +139,7 @@ on_text_custom_properties_clicked (GtkWidget *w, GladeXML *dialog)
 	gnome_vfs_mime_set_default_application ("text/plain", mime_app->id);
 	gnome_vfs_mime_application_free (mime_app);
 
-	gnome_vfs_application_registry_sync ();	
+	gnome_vfs_application_registry_sync ();
 }
 
 static void
@@ -155,7 +149,7 @@ on_text_default_viewer_toggle (GtkWidget *toggle, GladeXML *dialog)
 
 	old_action_type = gnome_vfs_mime_get_default_action_type ("text/plain");
 	new_action_type = GTK_TOGGLE_BUTTON (toggle)->active
-		? GNOME_VFS_MIME_ACTION_TYPE_APPLICATION 
+		? GNOME_VFS_MIME_ACTION_TYPE_APPLICATION
 		: GNOME_VFS_MIME_ACTION_TYPE_COMPONENT;
 
 	if (new_action_type == old_action_type)
@@ -188,46 +182,50 @@ generic_guard (GtkWidget *toggle, GtkWidget *widget)
 	}
 }
 
+static gboolean
+validate (GHashTable *unique, char const *name, char const *exec_name)
+{
+	if (NULL == g_hash_table_lookup (unique, name)) {
+		char *path = g_find_program_in_path (exec_name);
+		if (path != NULL) {
+			g_hash_table_insert (unique, (gpointer)name, (gpointer)exec_name);
+			g_free (path);
+			return TRUE;
+		}
+	}
+	return FALSE;
+}
+
 static void
 initialize_default_applications (void)
 {
         gint i;
+	GHashTable *unique;
 
 	text_editors = gnome_vfs_mime_get_all_applications ("text/plain");
 
-        for (i = 0; i < G_N_ELEMENTS (possible_browsers); i++ ) {
-		gchar *browsers = g_find_program_in_path (possible_browsers[i].executable_name);
-                if (browsers) {
-			possible_browsers[i].in_path = TRUE;
-			g_free(browsers);
-		}
-        }
-        for (i = 0; i < G_N_ELEMENTS (possible_mailers); i++ ) {
-		gchar *mailers = g_find_program_in_path (possible_mailers[i].executable_name);
-                if (mailers) {
-			possible_mailers[i].in_path = TRUE;
-			g_free(mailers);
-		}
-        }
-        for (i = 0; i < G_N_ELEMENTS (possible_help_viewers); i++ ) {
-                gchar *help_viewers = g_find_program_in_path (possible_help_viewers[i].executable_name);
-		if (help_viewers) {
-			possible_help_viewers[i].in_path = TRUE;
-			g_free (help_viewers);
-		}
-        }
-        for (i = 0; i < G_N_ELEMENTS (possible_terminals); i++ ) {
-                gchar *terminals = g_find_program_in_path (possible_terminals[i].exec);
-		if (terminals) {
-			possible_terminals[i].in_path = TRUE;
-			g_free (terminals);
-		}
-        }
+	unique = g_hash_table_new (g_str_hash, g_str_equal);
+        for (i = 0; i < G_N_ELEMENTS (possible_browsers); i++ )
+		possible_browsers[i].in_path = validate (unique,
+			possible_browsers[i].name, possible_browsers[i].executable_name);
+	g_hash_table_destroy (unique);
+
+	unique = g_hash_table_new (g_str_hash, g_str_equal);
+        for (i = 0; i < G_N_ELEMENTS (possible_mailers); i++ )
+		possible_mailers[i].in_path = validate (unique,
+			possible_mailers[i].name, possible_mailers[i].executable_name);
+	g_hash_table_destroy (unique);
+
+	unique = g_hash_table_new (g_str_hash, g_str_equal);
+        for (i = 0; i < G_N_ELEMENTS (possible_terminals); i++ )
+		possible_terminals[i].in_path = validate (unique,
+			possible_terminals[i].name, possible_terminals[i].exec);
+	g_hash_table_destroy (unique);
 }
 
 static void
 update_editor_sensitivity (GladeXML *dialog)
-{ 
+{
 	gboolean predefined = GTK_TOGGLE_BUTTON (WID ("text_select_radio"))->active;
 
 	gtk_widget_set_sensitive (WID ("text_select_combo"), predefined);
@@ -254,7 +252,7 @@ read_editor (GConfClient *client,
 
 		if (strcmp (mime_app->command, li_app->command) == 0 &&
 		    mime_app->requires_terminal == li_app->requires_terminal) {
-			gtk_entry_set_text (GTK_ENTRY (WID ("text_select_combo_entry")), mime_app->name);
+			gtk_entry_set_text (GTK_ENTRY (WID ("text_select_combo_entry")), _(mime_app->name));
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("text_custom_radio")), TRUE);
 			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("text_select_radio")), TRUE);
 			gnome_vfs_mime_application_free (mime_app);
@@ -262,7 +260,7 @@ read_editor (GConfClient *client,
 		}
         }
 
-	/* 
+	/*
 	 * the default editor wasn't set by us, and it wasn't in the
 	 * list.
 	 */
@@ -273,7 +271,7 @@ read_editor (GConfClient *client,
 	gnome_vfs_application_registry_save_mime_application (mime_app);
 
 	gnome_vfs_mime_set_default_application ("text/plain", mime_app->id);
-	gnome_vfs_application_registry_sync ();	
+	gnome_vfs_application_registry_sync ();
 
  read_editor_custom:
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("text_select_radio")), TRUE);
@@ -305,7 +303,7 @@ text_apply_editor (GtkWidget *entry,
 		mime_app = li->data;
 		if (! strcmp (mime_app->name, editor)) {
 			gnome_vfs_mime_set_default_application ("text/plain", mime_app->id);
-			gnome_vfs_application_registry_sync ();	
+			gnome_vfs_application_registry_sync ();
 			return;
 		}
 	}
@@ -333,19 +331,19 @@ text_apply_custom (GtkWidget *entry,
 	} else {
 		gnome_vfs_mime_set_default_application ("text/plain", mime_app->id);
 		gnome_vfs_mime_application_free (mime_app);
-		
-		gnome_vfs_application_registry_sync ();	
+
+		gnome_vfs_application_registry_sync ();
 	}
 
 	update_editor_sensitivity (dialog);
 }
 
-static void 
+static void
 setup_peditors (GConfClient *client,
 		GladeXML    *dialog)
 {
         GConfChangeSet *changeset = NULL;
-	
+
 	gconf_peditor_new_boolean (changeset, DEFAULT_APPS_KEY_BROWSER_NEEDS_TERM,
 				   WID ("web_custom_terminal_toggle"), NULL);
 	gconf_peditor_new_string  (changeset, DEFAULT_APPS_KEY_BROWSER_EXEC,
@@ -356,13 +354,6 @@ setup_peditors (GConfClient *client,
 	gconf_peditor_new_string  (changeset, DEFAULT_APPS_KEY_MAILER_EXEC,
 				   WID ("mail_custom_command_entry"), NULL);
 
-	gconf_peditor_new_boolean (changeset, DEFAULT_APPS_KEY_HELP_VIEWER_NEEDS_TERM,
-				   WID ("help_custom_terminal_toggle"), NULL);
-	gconf_peditor_new_boolean (changeset, DEFAULT_APPS_KEY_HELP_VIEWER_ACCEPTS_URLS,
-				   WID ("help_custom_url_toggle"), NULL);
-	gconf_peditor_new_string  (changeset, DEFAULT_APPS_KEY_HELP_VIEWER_EXEC,
-				   WID ("help_custom_command_entry"), NULL);
-
 	gconf_peditor_new_string  (changeset, DEFAULT_APPS_KEY_TERMINAL_EXEC,
 				   WID ("terminal_custom_command_entry"), NULL);
 	gconf_peditor_new_string  (changeset, DEFAULT_APPS_KEY_TERMINAL_EXEC_ARG,
@@ -391,7 +382,7 @@ read_browser (GConfClient *client,
 	for (i = 0; i < G_N_ELEMENTS (possible_browsers); i++ ) {
 		if (possible_browsers[i].in_path == FALSE)
 			continue;
-		
+
 		if (browser && strcmp (browser, possible_browsers[i].command) == 0 &&
 		    needs_term == possible_browsers[i].needs_term) {
 			gtk_entry_set_text (GTK_ENTRY (WID ("web_select_combo_entry")),
@@ -401,9 +392,15 @@ read_browser (GConfClient *client,
 			g_free (browser);
 			return;
 		}
-        }
-	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("web_select_radio")), TRUE);
-	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("web_custom_radio")), TRUE);
+	}
+	if (strlen(browser) != 0) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("web_select_radio")), TRUE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("web_custom_radio")), TRUE);
+	} else {
+		gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("web_select_radio")), TRUE);
+		gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("web_custom_radio")), TRUE);
+	}
+	
 	g_free (browser);
 }
 
@@ -429,7 +426,7 @@ read_mailer (GConfClient *client,
 	for (i = 0; i < G_N_ELEMENTS (possible_mailers); i++ ) {
 		if (possible_mailers[i].in_path == FALSE)
 			continue;
-		
+
 		if (mailer && strcmp (mailer, possible_mailers[i].command) == 0 &&
 		    needs_term == possible_mailers[i].needs_term) {
 			gtk_entry_set_text (GTK_ENTRY (WID ("mail_select_combo_entry")),
@@ -440,11 +437,17 @@ read_mailer (GConfClient *client,
 			return;
 		}
         }
-	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("mail_select_radio")), TRUE);
-	gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("mail_custom_radio")), TRUE);
+		
+	if (strlen(mailer) != 0) {
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("mail_select_radio")), TRUE);
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("mail_custom_radio")), TRUE);
+	} else {
+		gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("mail_select_radio")), TRUE);
+		gtk_toggle_button_set_inconsistent (GTK_TOGGLE_BUTTON (WID ("mail_custom_radio")), TRUE);
+	}
 	g_free (mailer);
 
-	
+
 }
 
 static void
@@ -483,74 +486,6 @@ mailer_setup_custom (GtkWidget *entry,
 	}
 }
 
-static void
-read_help_viewer (GConfClient *client,
-	     GladeXML    *dialog)
-{
-	GError *error = NULL;
-	gchar *help_viewer;
-	gboolean needs_term;
-	gboolean accepts_urls;
-	gint i;
-
-	needs_term = gconf_client_get_bool (client, DEFAULT_APPS_KEY_HELP_VIEWER_NEEDS_TERM, &error);
-	if (error) {
-		/* hp will shoot me -- I'll do this later. */
-		return;
-	}
-	accepts_urls = gconf_client_get_bool (client, DEFAULT_APPS_KEY_HELP_VIEWER_ACCEPTS_URLS, &error);
-	if (error) {
-		return;
-	}
-	help_viewer = gconf_client_get_string (client, DEFAULT_APPS_KEY_HELP_VIEWER_EXEC, &error);
-	if (error) {
-		return;
-	}
-
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_custom_terminal_toggle")), needs_term);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_custom_url_toggle")), accepts_urls);
-	gtk_entry_set_text (GTK_ENTRY (WID ("help_custom_command_entry")), help_viewer);
-
-	for (i = 0; i < G_N_ELEMENTS (possible_help_viewers); i++ ) {
-		if (possible_help_viewers[i].in_path == FALSE)
-			continue;
-		
-		if (help_viewer && strcmp (help_viewer, possible_help_viewers[i].executable_name) == 0 &&
-		    needs_term == possible_help_viewers[i].needs_term &&
-		    accepts_urls == possible_help_viewers[i].accepts_urls) {
-			gtk_entry_set_text (GTK_ENTRY (WID ("help_select_combo_entry")),
-					    _(possible_help_viewers[i].name));
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_custom_radio")), TRUE);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_select_radio")), TRUE);
-			g_free (help_viewer);
-			return;
-		}
-        }
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_select_radio")), TRUE);
-	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_custom_radio")), TRUE);
-	g_free (help_viewer);
-}
-
-static void
-help_setup_custom (GtkWidget *entry,
-		   GladeXML  *dialog)
-{
-	gint i;
-	const gchar *help_viewer = gtk_entry_get_text (GTK_ENTRY (entry));
-
-	for (i = 0; i < G_N_ELEMENTS (possible_help_viewers); i++ ) {
-		if (! strcmp (_(possible_help_viewers[i].name), help_viewer)) {
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_custom_terminal_toggle")),
-						      possible_help_viewers[i].needs_term);
-			gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (WID ("help_custom_url_toggle")),
-						      possible_help_viewers[i].accepts_urls);
-			gtk_entry_set_text (GTK_ENTRY (WID ("help_custom_command_entry")),
-					    possible_help_viewers[i].executable_name);
-			return;
-		}
-	}
-}
-
 
 static void
 read_terminal (GConfClient *client,
@@ -576,7 +511,7 @@ read_terminal (GConfClient *client,
 	for (i = 0; i < G_N_ELEMENTS (possible_terminals); i++ ) {
 		if (possible_terminals[i].in_path == FALSE)
 			continue;
-		
+
 		if (strcmp (exec?exec:"", possible_terminals[i].exec) == 0 &&
 		    strcmp (exec_arg?exec_arg:"", possible_terminals[i].exec_arg) == 0) {
 			gtk_entry_set_text (GTK_ENTRY (WID ("terminal_select_combo_entry")),
@@ -620,7 +555,6 @@ value_changed_cb (GConfClient *client,
 	if (strncmp (key, DEFAULT_APPS_KEY_MAILER_PATH, strlen (DEFAULT_APPS_KEY_MAILER_PATH)) == 0) {
 		gconf_client_set_bool (client, DEFAULT_APPS_KEY_MAILER_PATH"/enabled", TRUE, NULL);
 	} else if (strncmp (key, DEFAULT_APPS_KEY_BROWSER_PATH, strlen (DEFAULT_APPS_KEY_BROWSER_PATH)) == 0) {
-	} else if (strncmp (key, DEFAULT_APPS_KEY_HELP_VIEWER_PATH, strlen (DEFAULT_APPS_KEY_HELP_VIEWER_PATH)) == 0) {
 	} else if (strncmp (key, DEFAULT_APPS_KEY_TERMINAL_PATH, strlen (DEFAULT_APPS_KEY_TERMINAL_PATH)) == 0) {
 	}
 }
@@ -631,9 +565,7 @@ dialog_response (GtkDialog *widget,
 		 GladeXML  *dialog)
 {
 	if (response_id == GTK_RESPONSE_HELP)
-		capplet_help (GTK_WINDOW (widget),
-			"wgoscustlookandfeel.xml",
-			"goscustdoc-2");
+		capplet_help (GTK_WINDOW (widget), "user-guide.xml", "goscustdoc-2");
 	else
 		gtk_main_quit ();
 }
@@ -645,12 +577,13 @@ create_dialog (GConfClient *client)
 	GList *strings = NULL, *li;
 	gint i;
 	dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-default-applications-properties.glade", NULL, NULL);
-	
+	capplet_set_icon (WID ("default_applications_dialog"), "gnome-settings-default-applications");
+
 	setup_peditors (client, dialog);
 
 	/* Editors page */
 	for (li = text_editors; li; li = li->next) {
-		strings = g_list_append (strings, ((GnomeVFSMimeApplication *)li->data)->name);
+		strings = g_list_append (strings, _(((GnomeVFSMimeApplication *)li->data)->name));
 	}
 	if (strings) {
 		/* We have default editors */
@@ -668,7 +601,7 @@ create_dialog (GConfClient *client)
 	g_signal_connect (G_OBJECT (WID ("text_select_combo_entry")),
 			  "changed", G_CALLBACK (text_apply_editor),
 			  dialog);
-	g_signal_connect (WID ("text_custom_properties"), "clicked",			  
+	g_signal_connect (WID ("text_custom_properties"), "clicked",
 			  G_CALLBACK (on_text_custom_properties_clicked),
 			  dialog);
 	g_signal_connect (WID ("text_default_viewer_toggle"), "toggled",
@@ -743,41 +676,10 @@ create_dialog (GConfClient *client)
 			  WID ("mail_custom_vbox"));
 
 	read_mailer (client, dialog);
-	/* Help page */
-	
-	for (i = 0; i < G_N_ELEMENTS (possible_help_viewers); i++ ) {
-		if (possible_help_viewers[i].in_path)
-			strings = g_list_append (strings, _(possible_help_viewers[i].name));
-	}
-	if (strings) {
-		/* We have default help viewers */
-		gtk_combo_set_popdown_strings (GTK_COMBO(WID ("help_select_combo")), strings);
-		g_list_free (strings);
-		strings = NULL;
-	} else {
-		/* No default help viewers */
-		gtk_widget_set_sensitive (WID ("help_select_radio"), FALSE);
-	}
-
-	/* Source of command string */
-	g_object_set_data (G_OBJECT (WID ("help_select_radio")), "entry", WID ("help_select_combo_entry"));
-	/* Source of command string */
-	g_object_set_data (G_OBJECT (WID ("help_custom_radio")), "entry", WID ("help_custom_command_entry"));
-
-	g_signal_connect (G_OBJECT (WID ("help_select_combo_entry")),
-			  "changed", (GCallback) help_setup_custom,
-			  dialog);
-	g_signal_connect (G_OBJECT (WID ("help_select_radio")),
-			  "toggled", (GCallback) generic_guard,
-			  WID ("help_select_combo"));
-	g_signal_connect (G_OBJECT (WID ("help_custom_radio")),
-			  "toggled", (GCallback) generic_guard,
-			  WID ("help_custom_vbox"));
-	read_help_viewer (client, dialog);	
 
 
 	/* Terminal */
-	
+
 	for (i = 0; i < G_N_ELEMENTS (possible_terminals); i++ ) {
 		if (possible_terminals[i].in_path)
 			strings = g_list_append (strings, _(possible_terminals[i].name));
@@ -806,21 +708,20 @@ create_dialog (GConfClient *client)
 	g_signal_connect (G_OBJECT (WID ("terminal_custom_radio")),
 			  "toggled", (GCallback) generic_guard,
 			  WID ("terminal_custom_table"));
-	read_terminal (client, dialog);	
+	read_terminal (client, dialog);
 
 
 	g_signal_connect (G_OBJECT (client), "value-changed", (GCallback) value_changed_cb, dialog);
 	g_signal_connect (G_OBJECT (WID ("default_applications_dialog")), "response", (GCallback) dialog_response, dialog);
-	
+
 	gtk_widget_show (WID ("default_applications_dialog"));
 
 	return dialog;
 }
 
 static void
-get_legacy_settings (void) 
+get_legacy_settings (void)
 {
-
 }
 
 int
@@ -848,7 +749,6 @@ main (int argc, char **argv)
 	client = gconf_client_get_default ();
 
 	gconf_client_add_dir (client, "/desktop/gnome/applications/browser", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-	gconf_client_add_dir (client, "/desktop/gnome/applications/help_viewer", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	gconf_client_add_dir (client, "/desktop/gnome/applications/terminal", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
 	if (get_legacy) {
