@@ -1215,7 +1215,7 @@ setup_dialog (GladeXML *dialog)
   g_signal_connect (G_OBJECT (parent), "drag-leave", G_CALLBACK (gnome_theme_manager_drag_leave_cb), NULL);
   g_signal_connect (G_OBJECT (parent), "drag-data-received",G_CALLBACK (gnome_theme_manager_drag_data_received_cb), NULL);
 
-  capplet_set_icon (parent, "gnome-ccthemes.png");
+  capplet_set_icon (parent, "gnome-settings-theme");
 
   update_font_button_state (dialog);
   gtk_widget_show (parent);
